﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                               BFF class                                  |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 11-20-2006 (MM-DD-YYYY)            |
//|                        Last revised : 12-03-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Globalization;

using System.IO;

using Win32;

namespace Bff
{
    class BF
    {
        private const string zBff = @"zbff.dll";

        public const int ID_ZBFF = 777;
        public const int WM_USER = 1024;
        //      Bff_Clear                      = WM_USER + 1000; // Remove all items from Treeview.
        //      Bff_Refresh                    = WM_USER + 1001; // Force repaint.
        public const int Bff_Path              = WM_USER + 1002; // New path, wParam = size, lParam pointer to string.
        //      Bff_PathByPidl                 = WM_USER + 1003; // New path, wParam = CSIDL_...
        public const int Bff_JumpToPath        = WM_USER + 1004; // Set to path. Select item matching path.
        //      Bff_GetPath                    = WM_USER + 1005; // Returns path placed in provided buffer. wparam = size of empty buffer, lparam = pointer to buffer
        //      Bff_GetPathLen                 = WM_USER + 1006; // Use this to prepare a buffer.
        //      Bff_ExpandFirstItem            = WM_USER + 1007; // Open/close the first treeview item, wParam is switch
        //      Bff_BackColor                  = WM_USER + 1008; // Set color background. 
        
        public const int Bff_Ready             = WM_USER + 1009;
        public const int Bff_SelChange         = WM_USER + 1010;
        public const int Bff_RightClick        = WM_USER + 1011;
        public const int Bff_BeginLabelEdit    = WM_USER + 1012;
        public const int Bff_EndLabelEdit      = WM_USER + 1013;
        public const int Bff_MoveFolder        = WM_USER + 1014;
        public const int Bff_PrivateMessage    = WM_USER + 1015;

        public const int Bff_ColorBackTreeView = 1; // TreeView background  color
        public const int Bff_ColorTextTreeView = 2; // TreeView text and line color
        public const int Bff_ColorBackSelected = 3; // Selected item background color
        public const int Bff_ColorTextSelected = 4; // Selected item text color

        //public const string Bff_RegistryKey = @"HKEY_LOCAL_MACHINE\SOFTWARE\zBff";
        public const string Bff_RegistryKey = @"HKEY_CURRENT_USER\SOFTWARE\zBff";
        public const string Bff_RegistryPath = "DefaultPathName";

        [StructLayout(LayoutKind.Sequential)]
        public struct DRAGFOLDER
        {
            public int Dragfolder;
            public int Targetfolder;
        }

        private static IntPtr ctrlhandle = IntPtr.Zero;
        public static IntPtr CtrlHandle 
        {
            get { return ctrlhandle; }
            set { ctrlhandle = value; }
        }

        [DllImport(zBff)]
        public static extern bool RegisterBFFClass();

        [DllImport(zBff, EntryPoint = "Bff_SetProperty")]
        public static extern void SetProperty(IntPtr hWnd, int Item, int value);

        [DllImport(zBff)]
        public static extern string Bff_GetPath(IntPtr hWnd);

        [DllImport(zBff, EntryPoint = "Bff_ClickPath")]
        public static extern int ClickPath(IntPtr hTreeView);

        [DllImport(zBff, EntryPoint = "Bff_RenameFolder")]
        public static extern void RenameFolder(IntPtr hWnd, int Item, string zFolder);

        public static bool IsFR()
        {
            bool bRet = false; if (RegionInfo.CurrentRegion.Name == "FR") bRet = true;
            return bRet;
        }

        public static string NewFolder()
        {
            string sNewFolder = "New folder";
            if (IsFR()) sNewFolder = "Nouveau dossier";
            return sNewFolder;
        }

        public static string DeleteFolderMsg()
        {
            string Msg = "Do you really want to remove the folder '@' and send its whole content to tray?";
            if (IsFR()) Msg = "Voulez-vous vraiment supprimer le dossier '@' et envoyer l'ensemble de son contenu dans la corbeille ?";
            return Msg;
        }

        public static string DeleteFolderCaption()
        {
            string Msg = "Remove folder confirmation";
            if (IsFR()) Msg = "Confirmation de la suppression du dossier";
            return Msg;
        }

        public static string ErrorCaption()
        {
            string Msg = "Error";
            if (IsFR()) Msg = "Erreur";
            return Msg;
        }

        public static string NoFileAssociationMsg()
        {
            string Msg = "There is no program association for the file '@'.";
            if (IsFR()) Msg = "Il n'y a pas d'application associée au fichier '@'.";
            return Msg;
        }

        public static string MenuItemCreate()
        {
            string Msg = "Create new folder";
            if (IsFR()) Msg = "Créer un Nouveau dossier";
            return Msg;
        }
        public static string MenuItemRename()
        {
            string Msg = "Rename this folder";
            if (IsFR()) Msg = "Renommer le dossier";
            return Msg;
        }
        public static string MenuItemDelete()
        {
            string Msg = "Delete this folder";
            if (IsFR()) Msg = "Supprimer le dossier";
            return Msg;
        }

        public static string BTN_Yes()
        {
            string Msg = "Yes"; if (IsFR()) Msg = "Oui";
            return Msg;
        }

        public static string BTN_No()
        {
            string Msg = "No"; if (IsFR()) Msg = "Non";
            return Msg;
        }

        public static string BTN_Ok()
        {
            string Msg = "Ok"; if (IsFR()) Msg = "Ok";
            return Msg;
        }

        public static IntPtr GetTreeViewHandle()
        {
            return Api.GetDlgItem(CtrlHandle, ID_ZBFF);
        }

        public static string GetPath()
        {
            return Bff_GetPath(GetTreeViewHandle());
        }

        [DllImport(zBff, EntryPoint = "Bff_Path")]
        public static extern int ResetPath(IntPtr hTreeView, string PathAsText, int PathAsPIDL, int ExpandItem);

        [DllImport(zBff, EntryPoint = "Bff_JumpToPath")]
        public static extern int JumpToPath(IntPtr hTreeView, string sFolderName);

        [DllImport(zBff, EntryPoint = "Bff_ClickFolder")]
        public static extern string ClickFolder(IntPtr hTreeView, int FolderItem);

        [DllImport(zBff, EntryPoint = "Bff_MoveToTray")]
        public static extern int MoveToTray(string sFile);

        [DllImport(zBff, EntryPoint = "Bff_FolderMove")]
        public static extern int FolderMove(string sDragfolder, string sTargetfolder);

        public static bool IsValidName(string sNameToCheck)
        {
            string sNewFolder = Path.GetTempPath() + sNameToCheck;
            DirectoryInfo di = new DirectoryInfo(sNewFolder);
            bool Ok = true;
            try
            {// Determine whether the directory exists.
                if (di.Exists)
                {// Indicate that the directory already exists.
                    Ok = false;
                    return Ok;
                }
                else
                {// Try to create the directory.
                    di.Create();
                }
            }
            catch //(Exception er)
            {// Error
                Ok = false;
            }
            finally {}
            if (Ok) Directory.Delete(sNewFolder);
            return Ok;
        }

        public static bool RenameFolder(string sWasOldName, string sWasNewName)
        {
            bool Ok = true;
            try
            { Directory.Move(sWasOldName, sWasNewName); }
            catch //(Exception er)
            {// Error
                Ok = false;
            }
            finally {}
            return Ok;
        }

        public static string GetTreeViewFolder(int FolderItem)
        {
            if (FolderItem == 0)
            {
                FolderItem = (int)ClickPath(GetTreeViewHandle());
            }
            return ClickFolder(GetTreeViewHandle(), FolderItem);
        }

        [DllImport(zBff)]
        public static extern IntPtr zBrowser(
            uint ExStyle,
            string zDefaultPathName,
            uint Style,
            int x,
            int y,
            int w,
            int h,
            IntPtr parent,
            int ID);

        //public static string GetTempFolder()
        //{
        //    string TempFolder = System.IO.Path.GetTempPath() + "zBff_temp";
        //    Directory.CreateDirectory(TempFolder);
        //    return TempFolder;
        //}

    }
}
