﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                               zExplorer                                  |
//|                          File Browser utility                            |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 11-20-2006 (MM-DD-YYYY)            |
//|                        Last revised : 12-03-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Diagnostics;
using Microsoft.Win32;

using Win32;
using Bff;

namespace zExplorer
{
    public partial class MAIN_Form : Form
    {
        private string zDefaultPathName = "";

        public MAIN_Form()
        {
            InitializeComponent();
        }

        #region Create and set ZBFF properties + Init/Resize section.
        private void MAIN_Form_Load(object sender, EventArgs e)
        {
            // Retrieve the last path being used from the registry
            zDefaultPathName = (string)Registry.GetValue(BF.Bff_RegistryKey, BF.Bff_RegistryPath, "");

            BF.CtrlHandle = BF.zBrowser(Api.WS_EX_STATICEDGE,
                                     zDefaultPathName,
                                     Api.WS_CHILD | Api.WS_VISIBLE,
                                     ZBFF_Container.Location.X,
                                     ZBFF_Container.Location.Y,
                                     ZBFF_Container.Width,
                                     ZBFF_Container.Height,
                                     this.Handle,
                                     BF.ID_ZBFF);

            BF.SetProperty(BF.CtrlHandle, BF.Bff_ColorBackTreeView, Api.RGB(255, 255, 255)); // TreeView background  color
            BF.SetProperty(BF.CtrlHandle, BF.Bff_ColorTextTreeView, Api.RGB(0, 0, 0));       // TreeView text and line color
            BF.SetProperty(BF.CtrlHandle, BF.Bff_ColorBackSelected, Api.RGB(178, 180, 191)); // Selected item background color
            BF.SetProperty(BF.CtrlHandle, BF.Bff_ColorTextSelected, Api.RGB(0, 0, 0));       // Selected item text color

            UpdateStatusMessage();
        }

        private void ZBFF_Resize(object sender, EventArgs e)
        {
            Api.MoveWindow(BF.CtrlHandle, ZBFF_Container.Location.X, ZBFF_Container.Location.Y, ZBFF_Container.Width, ZBFF_Container.Height, true);
        }
        private void InitTreeViewFolder(string sFolderName)
        {
            IntPtr hTreeView = BF.GetTreeViewHandle();
            // Reset TreeView content
            BF.ResetPath(hTreeView, "", 0, 1);
            // Set active folder
            BF.JumpToPath(hTreeView, sFolderName);
        }

        private string CompletePath(string UsePath)
        {
            return UsePath.TrimEnd('\\') + @"\";
        }
        #endregion

        #region Update registry
        private bool UpdateRegistry(string Usepath)
        {
            bool DoIt = false;
            // Save current folder name in registry
            Registry.SetValue(BF.Bff_RegistryKey, BF.Bff_RegistryPath, Usepath);

            // Check if registry has been correctly updated and setup DoIt accordingly
            if (Usepath == (string)Registry.GetValue(BF.Bff_RegistryKey, BF.Bff_RegistryPath, ""))
            {
                UpdateStatusMessage();
                DoIt = true;
            }
            return DoIt;
        }
        #endregion

        #region Main ZBFF TreeView command section
        unsafe protected override void WndProc(ref Message m)
        {// ZBFF message command section
            switch (m.Msg)
            {
                case Api.WM_COMMAND:
                    if (Api.LoWrd((uint)m.WParam) == BF.ID_ZBFF)
                    {
                        bool DoIt = false;
                        int BffCommand = (int)Api.HiWrd((uint)m.WParam);
                        switch (BffCommand)
                        {
                            case BF.Bff_Ready:
                                DoIt = true;
                                break;

                            case BF.Bff_SelChange:
                                DoIt = true;
                                break;

                            case BF.Bff_RightClick:
                                string Usepath = BF.GetPath();
                                if (CompletePath(Usepath).Length > 3)
                                {// Create popup contextual menu
                                    ShowTreeViewContextMenu();
                                }
                                break;

                            case BF.Bff_BeginLabelEdit:
                                break;

                            case BF.Bff_EndLabelEdit:
                                IntPtr hTreeView = BF.GetTreeViewHandle();
                                IntPtr hEdit = (IntPtr)Api.SendMessage(hTreeView, Api.TVM_GETEDITCONTROL, 0, 0);
                                if (hEdit != IntPtr.Zero)
                                {
                                    string sNewName = Api.GetWindowText(hEdit).Trim();
                                    if (sNewName.Length > 0)
                                    {
                                        string sWasNewName = sNewName;
                                        if (BF.IsValidName(sNewName))
                                        {
                                            sNewName = sNewName.ToLower();
                                            string sWasOldName = BF.GetTreeViewFolder(0);
                                            string sOldName = sWasOldName.ToLower();
                                            string sRootPath = CompletePath(Path.GetDirectoryName(sWasOldName));
                                            sNewName = sRootPath + sNewName;
                                            if (!Directory.Exists(sNewName))
                                            {
                                                sWasNewName = sRootPath + sWasNewName;
                                                if (BF.RenameFolder(sWasOldName, sWasNewName))
                                                {
                                                    UpdateRegistry(sWasNewName);
                                                    int nItem = BF.ClickPath(hTreeView);
                                                    BF.RenameFolder(hTreeView, nItem, sWasNewName);
                                                }
                                            }
                                        }
                                    }
                                }
                                Api.SetWindowLong(hTreeView, Api.GWL_STYLE, (Api.GetWindowLong(hTreeView, Api.GWL_STYLE) | Api.TVS_EDITLABELS));
                                break;

                            case BF.Bff_MoveFolder:
                                // Unsafe code to retrieve DRAGFOLDER structure parameters
                                BF.DRAGFOLDER* BDF = (BF.DRAGFOLDER*)m.LParam;
                                string sDragfolder = BF.GetTreeViewFolder(BDF->Dragfolder);
                                string sTargetfolder = BF.GetTreeViewFolder(BDF->Targetfolder);

                                if (BF.FolderMove(sDragfolder, sTargetfolder) == 0)
                                {
                                    string sFolderName = CompletePath(sTargetfolder) + Path.GetFileName(sDragfolder);
                                    InitTreeViewFolder(sFolderName);
                                }
                                break;
                        }

                        if (DoIt)
                        {
                            UpdateTable(BffCommand);
                        }

                    }
                    break;
            }

            base.WndProc(ref m);
        }
        #endregion

        #region DataGridView event/command section and cell formating
        private void GRID_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {// Start EXE or Document
            if (e.RowIndex > -1)
            {
                string Usepath = BF.GetPath();
                string FullPathName = CompletePath(Usepath) + (string)GRID_Content.Rows[e.RowIndex].Cells[1].Value;
                if (File.Exists(FullPathName))
                {// Check for file association.
                    try
                    {// We must first set the current directory to the selected folder.
                        Directory.SetCurrentDirectory(Usepath);
                        Process.Start(FullPathName);
                    }
                    catch //(Exception er)
                    {// Error No file association
                        string Message = BF.NoFileAssociationMsg().Replace("@", ((string)GRID_Content.Rows[e.RowIndex].Cells[1].Value).ToUpper());
                        Form MsgBox = new MsgBoxError(BF.ErrorCaption(), Message);
                        MsgBox.ShowDialog(this);
                        MsgBox.Dispose();
                    }
                    finally { }
                }
            }
        }

        private string FormatSize(Int64 fSize)
        {
            if (fSize > 0) fSize = ((fSize + 1024) / 1024);
            return fSize.ToString("N0");
        }

        private string FormatDate(DateTime fDate)
        {
            return fDate.ToShortDateString().ToString();
        }

        private void ClearFolderContent()
        {
            GRID_Content.Rows.Clear();
        }

        private void UpdateTable(int BffCommand)
        {
            string Usepath = BF.GetPath();
            ClearFolderContent();

            if (Usepath != null)
            {
                Usepath = CompletePath(Usepath);
                string FileFound, FileExt, FullName;
                string Dot = ".";

                int Count = 0;
                foreach (string f in Directory.GetFiles(Usepath))
                {
                    Count++;
                    FileFound = f.Replace(Usepath, "");
                    int So = FileFound.LastIndexOf(Dot);
                    FileExt = ""; if (So > -1) FileExt = FileFound.Substring(So, FileFound.Length - So).ToLower().TrimStart('.');

                    FullName = Usepath + FileFound;
                    FileInfo fi = new FileInfo(FullName);
                    Bitmap IconBitmap = Icon.ExtractAssociatedIcon(FullName).ToBitmap();

                    GRID_Content.Rows.Add(new object[] 
                        { 
                            IconBitmap, 
                            FileFound, 
                            FormatSize(fi.Length) + " Kb", 
                            FormatDate(fi.LastWriteTime), 
                            FileExt.TrimStart('.') 
                        });

                }

                if (BffCommand == BF.Bff_SelChange)
                {
                    UpdateRegistry(Usepath);
                }
            }
        }
        #endregion

        #region TreeView popup menu
        private void ShowTreeViewContextMenu()
        {
            ContextMenuStrip zBffMenu = new ContextMenuStrip();
            zBffMenu.Items.Add(BF.MenuItemCreate(), null, OnNewFolder);
            zBffMenu.Items.Add(BF.MenuItemRename(), null, OnRenameFolder);
            zBffMenu.Items.Add(BF.MenuItemDelete(), null, OnDeleteFolder);
            Point pt = new Point();
            Api.GetCursorPos(ref pt);
            zBffMenu.Show(pt);
        }

        private void OnNewFolder(object sender, EventArgs e)
        {
            string sNewFolder = CompletePath(BF.GetTreeViewFolder(0)) + BF.NewFolder();
            DirectoryInfo di = new DirectoryInfo(sNewFolder);
            bool Ok = true;
            try
            {// Determine whether the directory exists.
                if (di.Exists)
                {// Indicate that the directory already exists.
                    Ok = false;
                }
                else
                {// Try to create the directory.
                    di.Create();
                }
            }
            catch //(Exception er)
            {
                Ok = false;
            }
            finally {}

            if (Ok) //Directory.CreateDirectory(sNewFolder))
            {
                // Reset TreeView content and move to folder root
                InitTreeViewFolder(sNewFolder);
                // Update grid view
                UpdateTable(BF.Bff_SelChange);
                IntPtr hTreeView = BF.GetTreeViewHandle();
                // Switch to edit mode to let the user change the default name
                Api.SetWindowLong(hTreeView, Api.GWL_STYLE, (Api.GetWindowLong(hTreeView, Api.GWL_STYLE) | Api.TVS_EDITLABELS));
                Api.SendMessage(hTreeView, Api.TVM_EDITLABEL, 0, BF.ClickPath(hTreeView));
            }
        }

        private void OnRenameFolder(object sender, EventArgs e)
        {// Rename the selected folder
            IntPtr hTreeView = BF.GetTreeViewHandle();
            Api.SetWindowLong(hTreeView, Api.GWL_STYLE, (Api.GetWindowLong(hTreeView, Api.GWL_STYLE) | Api.TVS_EDITLABELS));
            Api.SendMessage(hTreeView, Api.TVM_EDITLABEL, 0, BF.ClickPath(hTreeView));
        }

        private void OnDeleteFolder(object sender, EventArgs e)
        {// Delete selected folder
            string sFolderName = BF.GetTreeViewFolder(0);
            string Message = BF.DeleteFolderMsg().Replace("@", Path.GetFileName(sFolderName).ToUpper());
            Form MsgBox = new MsgBoxYesNo(BF.DeleteFolderCaption(), Message);
            if (MsgBox.ShowDialog(this) == DialogResult.Yes)
            {
                BF.MoveToTray(sFolderName);

                sFolderName = sFolderName.Substring(0, sFolderName.LastIndexOf(@"\"));
                // Reset TreeView content and move to folder root
                InitTreeViewFolder(sFolderName);
            }
            MsgBox.Dispose();
        }
        #endregion

        #region Custom Splitter
        private bool SplitVertMoving = false;
        private int xSplitVert = 0;
        private int ySplitVert = 0;
        private void SPLIT_Vert_MouseDown(object sender, MouseEventArgs e)
        {
            SplitVertMoving = true;
            xSplitVert = e.X; ySplitVert = e.Y;
        }

        private void SPLIT_Vert_MouseMove(object sender, MouseEventArgs e)
        {// This is the core section of our custom splitter.
            if (SplitVertMoving)
            {
                if (e.X != xSplitVert)
                {
                    Rectangle r = this.ClientRectangle;
                    if ((SPLIT_Vert.Location.X + e.X > 80) && (SPLIT_Vert.Location.X + e.X <= r.Right - SPLIT_Vert.Width))
                    {
                        //this.SuspendLayout();
                        ZBFF_Container.Width += e.X;
                        SPLIT_Vert.Location = new Point(SPLIT_Vert.Location.X + e.X, 0);
                        GRID_Content.Location = new Point(GRID_Content.Location.X + e.X, 0);
                        GRID_Content.Width = GRID_Content.Width - e.X;
                        // UpdateWindow is much faster than Control.Invalidate(); 
                        Api.UpdateWindow(GRID_Content.Handle);
                        //unsafe
                        //{
                        //    int[] MyRect = { 0, 0, GRID_Content.Width, GRID_Content.Height };
                        //    fixed (int* pRect = MyRect)
                        //    Api.InvalidateRect(GRID_Content.Handle, (int)pRect, true);
                        //    Api.UpdateWindow(GRID_Content.Handle);
                        //}
                        Api.UpdateWindow(BF.GetTreeViewHandle());
                        //this.ResumeLayout();
                    }
                }
                xSplitVert = e.X; ySplitVert = e.Y;
            }
        }

        private void SPLIT_Vert_MouseUp(object sender, MouseEventArgs e)
        {
            SplitVertMoving = false;
            xSplitVert = 0; ySplitVert = 0;
        }

        private void ZBFF_Restore(object sender, EventArgs e)
        {
            Rectangle r = this.ClientRectangle;
            int rsw = r.Right - SPLIT_Vert.Width;
            if (SPLIT_Vert.Location.X > rsw)
            {// Check if controls are not out of client bounds
                int rsh = r.Bottom - STATUS_Main.Height;
                ZBFF_Container.Width = rsw;
                ZBFF_Container.Height = rsh;

                SPLIT_Vert.Location = new Point(rsw, 0);
                SPLIT_Vert.Height = rsh;
                GRID_Content.Location = new Point(r.Right, 0);
                GRID_Content.Width = 0;
                GRID_Content.Height = rsh;
                Api.UpdateWindow(BF.GetTreeViewHandle());
            }
        }
        #endregion

        #region Display status message
        private void UpdateStatusMessage()
        {// Refresh status bar with the current folder name
            STATUS_Msg.Text = BF.GetPath();
        }
        #endregion

    }
}